@@ -19,7 +19,7 @@ import pysam
 from py_descriptive_statistics import Enum as Stats
 
 from bcbio import utils, broad
-from bcbio.log import logger
+from bcbio.log import logger, setup_local_logging
 from bcbio.distributed.messaging import parallel_runner
 from bcbio.distributed.split import parallel_split_combine
 from bcbio.distributed.transaction import file_transaction
@@ -27,6 +27,7 @@ from bcbio.pipeline import shared
 
 def parallel_callable_loci(in_bam, ref_file, config):
     num_cores = config["algorithm"].get("num_cores", 1)
+    config["algorithm"]["memory_adjust"] = {"direction": "decrease", "magnitude": 2}
     data = {"work_bam": in_bam, "sam_ref": ref_file, "config": config}
     parallel = {"type": "local", "cores": num_cores, "module": "bcbio.distributed"}
     runner = parallel_runner(parallel, {}, config)
@@ -53,6 +54,10 @@ def calc_callable_loci(data, region=None, out_file=None):
     http://www.broadinstitute.org/gatk/gatkdocs/
     org_broadinstitute_sting_gatk_walkers_coverage_CallableLoci.html
     """
+    if data["config"].get("parallel", {}).get("log_queue"):
+        handler = setup_local_logging(data["config"], data["config"]["parallel"])
+    else:
+        handler = None
     broad_runner = broad.runner_from_config(data["config"])
     if out_file is None:
         out_file = "%s-callable.bed" % os.path.splitext(data["work_bam"])[0]
@@ -78,6 +83,8 @@ def calc_callable_loci(data, region=None, out_file=None):
                         if tregion.chrom == region:
                             out_handle.write("%s\t%s\t%s\tNO_COVERAGE\n" %
                                              (tregion.chrom, tregion.start, tregion.stop))
+    if handler and hasattr(handler, "close"):
+        handler.close()
     return [{"callable_bed": out_file, "config": data["config"], "work_bam": data["work_bam"]}]
 
 def sample_callable_bed(bam_file, ref_file, config):
