@@ -3,6 +3,7 @@
   Picard -- BAM manipulation and analysis library.
   GATK -- Next-generation sequence processing.
 """
+import copy
 import os
 import subprocess
 from contextlib import closing
@@ -18,12 +19,34 @@ class BroadRunner:
     def __init__(self, picard_ref, gatk_dir, config):
         resources = config_utils.get_resources("gatk", config)
         self._jvm_opts = resources.get("jvm_opts", ["-Xms750m", "-Xmx2g"])
-        self._picard_ref = picard_ref
-        self._gatk_dir = gatk_dir or picard_ref
+        self._picard_ref = config_utils.expand_path(picard_ref)
+        self._gatk_dir = config_utils.expand_path(gatk_dir) or config_utils.expand_path(picard_ref)
         self._config = config
         self._resources = resources
         self._gatk_version = None
 
+    def _context_jvm_opts(self, config):
+        """Establish JVM opts, adjusting memory for the context if needed.
+
+        This allows using less or more memory for highly parallel or multicore
+        supporting processes, respectively.
+        """
+        jvm_opts = []
+        memory_adjust = config["algorithm"].get("memory_adjust", {})
+        for opt in self._jvm_opts:
+            if opt.startswith(("-Xmx", "-Xms")):
+                arg = opt[:4]
+                modifier = opt[-1:]
+                amount = int(opt[4:-1])
+                if memory_adjust.get("direction") == "decrease":
+                    amount = amount / memory_adjust.get("magnitude", 1)
+                elif memory_adjust.get("direction") == "increase":
+                    amount = amount * memory_adjust.get("magnitude", 1)
+                opt = "{arg}{amount}{modifier}".format(arg=arg, amount=amount,
+                                                       modifier=modifier)
+            jvm_opts.append(opt)
+        return jvm_opts
+
     def run_fn(self, name, *args, **kwds):
         """Run pre-built functionality that used Broad tools by name.
 
@@ -81,6 +104,7 @@ class BroadRunner:
         gatk_jar = self._get_jar("GenomeAnalysisTK", ["GenomeAnalysisTKLite"])
         local_args = []
         cores = self._config["algorithm"].get("num_cores", 1)
+        config = copy.deepcopy(self._config)
         if cores and int(cores) > 1:
             atype_index = params.index("-T") if params.count("-T") > 0 \
                           else params.index("--analysis_type")
@@ -89,12 +113,28 @@ class BroadRunner:
                 params.extend(["-nt", str(cores)])
             elif prog in support_nct:
                 params.extend(["-nct", str(cores)])
+                if config["algorithm"].get("memory_adjust") is None:
+                    config["algorithm"]["memory_adjust"] = {"direction": "increase",
+                                                            "magnitude": int(cores) // 2}
         if self.get_gatk_version() > "1.9":
             if len([x for x in params if x.startswith(("-U", "--unsafe"))]) == 0:
                 params.extend(["-U", "LENIENT_VCF_PROCESSING"])
             params.extend(["--read_filter", "BadCigar", "--read_filter", "NotPrimaryAlignment"])
         local_args.append("-Djava.io.tmpdir=%s" % tmp_dir)
-        return ["java"] + self._jvm_opts + local_args + \
+        return ["java"] + self._context_jvm_opts(config) + local_args + \
+          ["-jar", gatk_jar] + [str(x) for x in params]
+
+    def cl_mutect(self, params, tmp_dir):
+
+        """Define parameters to run the mutect paired algorithm."""
+
+        gatk_jar = self._get_jar("muTect")
+        local_args = []
+        cores = self._config["algorithm"].get("num_cores", 1)
+        config = copy.deepcopy(self._config)
+
+        local_args.append("-Djava.io.tmpdir=%s" % tmp_dir)
+        return ["java"] + self._context_jvm_opts(config) + local_args + \
           ["-jar", gatk_jar] + [str(x) for x in params]
 
     def run_gatk(self, params, tmp_dir=None):
@@ -107,6 +147,15 @@ class BroadRunner:
             prog = cl[atype_index + 1]
             do.run(cl, "GATK: {0}".format(prog), None)
 
+    def run_mutect(self, params, tmp_dir=None):
+
+        with curdir_tmpdir() as local_tmp_dir:
+            if tmp_dir is None:
+                tmp_dir = local_tmp_dir
+            cl = self.cl_mutect(params, tmp_dir)
+            prog = "MuTect"
+            do.run(cl, "MuTect: {0}".format(prog), None)
+
     def get_gatk_version(self):
         """Retrieve GATK version, handling locally and config cached versions.
         Calling version can be expensive due to all the startup and shutdown
@@ -169,17 +218,15 @@ class BroadRunner:
         dirs = []
         for bdir in [self._gatk_dir, self._picard_ref]:
             dirs.extend([bdir,
-                         os.path.join(bdir, os.pardir, "gatk"),
-                         os.path.join(bdir, "dist"),
-                         os.path.join(bdir, "GATK"),
-                         os.path.join(bdir, "GATK", "dist"),
-                         os.path.join(bdir, "Picard-private", "dist")])
+                         os.path.join(bdir, os.pardir, "gatk")])
         if alts is None: alts = []
         for check_cmd in [command] + alts:
             for dir_check in dirs:
-                check_file = os.path.join(dir_check, "%s.jar" % check_cmd)
-                if os.path.exists(check_file):
+                try:
+                    check_file = config_utils.get_jar(command, dir_check)
                     return check_file
+                except ValueError:
+                    pass
         raise ValueError("Could not find jar %s in %s:%s" % (command, self._picard_ref, self._gatk_dir))
 
 def _get_picard_ref(config):
@@ -205,7 +252,7 @@ def _get_picard_ref(config):
         picard = config_utils.get_program("picard", config, "dir")
     return picard
 
-def runner_from_config(config):
+def runner_from_config(config, program="gatk"):
     return BroadRunner(_get_picard_ref(config),
-                       config_utils.get_program("gatk", config, "dir"),
+                       config_utils.get_program(program, config, "dir"),
                        config)
