@@ -9,18 +9,18 @@ Cluster implementation from ipython-cluster-helper:
 https://github.com/roryk/ipython-cluster-helper
 """
 import os
+import contextlib
 import copy
 
 from bcbio import utils
-from bcbio.log import logger
+from bcbio.log import logger, get_log_dir
 from bcbio.pipeline import config_utils
 
 from cluster_helper import cluster as ipython_cluster
 
 def dictadd(orig, k, v):
     """Imitates immutability by adding a key/value to a new dictionary.
-    Works around not being able to deepcopy view objects; can remove this
-    once we create views on demand.
+    Works around not being able to deepcopy view objects.
     """
     view = orig.pop("view", None)
     new = copy.deepcopy(orig)
@@ -30,6 +30,14 @@ def dictadd(orig, k, v):
         new["view"] = view
     return new
 
+def dictdissoc(orig, k):
+    """Imitates immutability: create a new dictionary with the key dropped.
+    """
+    v = orig.pop(k, None)
+    new = copy.deepcopy(orig)
+    orig[k] = v
+    return new
+
 def _get_resource_programs(fn, algs):
     """Retrieve programs used in analysis based on algorithm configurations.
 
@@ -46,20 +54,23 @@ def _get_resource_programs(fn, algs):
         if prog in used_progs:
             yield prog
 
-def find_cores_per_job(fn, parallel, items, config):
+def find_cores_per_job(fns, parallel, items, config, multiplier=1):
     """Determine cores and workers to use for this stage based on function metadata.
+    multiplier specifies the number of regions items will be split into during
+    processing.
     """
     all_cores = [1]
     algs = [get_algorithm_config(x) for x in items]
-    for prog in _get_resource_programs(fn, algs):
-        resources = config_utils.get_resources(prog, config)
-        cores = resources.get("cores")
-        if cores:
-            all_cores.append(cores)
+    for fn in fns:
+        for prog in _get_resource_programs(fn, algs):
+            resources = config_utils.get_resources(prog, config)
+            cores = resources.get("cores")
+            if cores:
+                all_cores.append(cores)
     cores_per_job = max(all_cores)
     total = parallel["cores"]
     if total > cores_per_job:
-        return min(total // cores_per_job, len(items)), cores_per_job
+        return min(total // cores_per_job, len(items) * multiplier), cores_per_job
     else:
         return 1, total
 
@@ -105,17 +116,62 @@ def add_cores_to_config(args, cores_per_job, parallel=None):
     if is_nested_config_arg(new_arg):
         new_arg["config"]["algorithm"]["num_cores"] = int(cores_per_job)
         if parallel:
-            new_arg["config"]["parallel"] = parallel
+            new_arg["config"]["parallel"] = dictdissoc(parallel, "view")
     elif is_std_config_arg(new_arg):
         new_arg["algorithm"]["num_cores"] = int(cores_per_job)
         if parallel:
-            new_arg["parallel"] = parallel
+            new_arg["parallel"] = dictdissoc(parallel, "view")
     else:
         raise ValueError("Unexpected configuration dictionary: %s" % new_arg)
     args = list(args)[:]
     args[new_i] = new_arg
     return args
 
+def _view_from_parallel(parallel, work_dir, config):
+    """Translate parallel map into options for a cluster view.
+    """
+    profile_dir = utils.safe_makedir(os.path.join(work_dir, get_log_dir(config), "ipython"))
+    return ipython_cluster.cluster_view(parallel["scheduler"].lower(), parallel["queue"],
+                                        parallel["num_jobs"], parallel["cores_per_job"],
+                                        profile=profile_dir, start_wait=parallel["timeout"],
+                                        extra_params={"resources": parallel["resources"]},
+                                        retries=parallel.get("retries"))
+
+def _get_ipython_fn(fn_name, parallel):
+    return getattr(__import__("{base}.ipythontasks".format(base=parallel["module"]),
+                            fromlist=["ipythontasks"]),
+                   fn_name)
+
+@contextlib.contextmanager
+def global_parallel(parallel, name, fn_names, items, work_dir, config,
+                    multiplier=1):
+    """Add an IPython cluster to be used for multiple remote functions.
+
+    Allows sharing of a single cluster across multiple functions with
+    identical resource requirements. Falls back into local execution for
+    non-distributed clusters or completed jobs.
+    """
+    checkpoint_dir = utils.safe_makedir(os.path.join(work_dir, "checkpoints_ipython"))
+    checkpoint_file = os.path.join(checkpoint_dir, "global-%s.done" % name)
+    try:
+        if parallel["type"] != "ipython" or os.path.exists(checkpoint_file):
+            yield parallel
+        else:
+            items = [x for x in items if x is not None]
+            num_jobs, cores_per_job = find_cores_per_job([_get_ipython_fn(x, parallel) for x in fn_names],
+                                                         parallel, items, config, multiplier=multiplier)
+            parallel = dictadd(parallel, "cores_per_job", cores_per_job)
+            parallel = dictadd(parallel, "num_jobs", num_jobs)
+            with _view_from_parallel(parallel, work_dir, config) as view:
+                parallel["view"] = view
+                yield parallel
+    except:
+        raise
+    else:
+        parallel["view"] = None
+        with open(checkpoint_file, "w") as out_handle:
+            out_handle.write("done\n")
+
 def runner(parallel, fn_name, items, work_dir, config):
     """Run a task on an ipython parallel cluster, allowing alternative queue types.
 
@@ -124,17 +180,20 @@ def runner(parallel, fn_name, items, work_dir, config):
 
     A checkpoint directory keeps track of finished tasks, avoiding spinning up clusters
     for sections that have been previous processed.
+
+    The parallel input function can contain a pre-created view on an
+    existing Ipython cluster, in which case this will be reused
+    instead of creating a new cluster.
     """
     out = []
     checkpoint_dir = utils.safe_makedir(os.path.join(work_dir, "checkpoints_ipython"))
     checkpoint_file = _get_checkpoint_file(checkpoint_dir, fn_name)
-    fn = getattr(__import__("{base}.ipythontasks".format(base=parallel["module"]),
-                            fromlist=["ipythontasks"]),
-                 fn_name)
+    fn = _get_ipython_fn(fn_name, parallel)
     items = [x for x in items if x is not None]
-    num_jobs, cores_per_job = find_cores_per_job(fn, parallel, items, config)
-    parallel = dictadd(parallel, "cores_per_job", cores_per_job)
-    parallel = dictadd(parallel, "num_jobs", num_jobs)
+    if parallel.get("view") is None:
+        num_jobs, cores_per_job = find_cores_per_job([fn], parallel, items, config)
+        parallel = dictadd(parallel, "cores_per_job", cores_per_job)
+        parallel = dictadd(parallel, "num_jobs", num_jobs)
     # already finished, run locally on current machine to collect details
     if os.path.exists(checkpoint_file):
         logger.info("ipython: %s -- local; checkpoint passed" % fn_name)
@@ -147,13 +206,16 @@ def runner(parallel, fn_name, items, work_dir, config):
     else:
         logger.info("ipython: %s" % fn_name)
         if len(items) > 0:
-            items = [add_cores_to_config(x, cores_per_job, parallel) for x in items]
-            with ipython_cluster.cluster_view(parallel["scheduler"].lower(), parallel["queue"],
-                                              parallel["num_jobs"], parallel["cores_per_job"],
-                                              profile=parallel["profile"]) as view:
-                for data in view.map_sync(fn, items, track=False):
+            items = [add_cores_to_config(x, parallel["cores_per_job"], parallel) for x in items]
+            if parallel.get("view"):
+                for data in parallel["view"].map_sync(fn, items, track=False):
                     if data:
                         out.extend(data)
+            else:
+                with _view_from_parallel(parallel, work_dir, config) as view:
+                    for data in view.map_sync(fn, items, track=False):
+                        if data:
+                            out.extend(data)
     with open(checkpoint_file, "w") as out_handle:
         out_handle.write("done\n")
     return out
