@@ -6,7 +6,8 @@ from IPython.parallel import require
 
 from bcbio.distributed import ipython
 from bcbio.pipeline import sample, lane, qcsummary, shared, variation
-from bcbio.variation import bamprep, realign, genotype, ensemble, multi, population, recalibrate
+from bcbio.variation import (bamprep, realign, genotype, ensemble, multi, population,
+                             recalibrate, validate, vcfutils)
 from bcbio.log import logger, setup_local_logging
 
 @contextlib.contextmanager
@@ -46,7 +47,7 @@ def trim_lane(*args):
 def process_alignment(*args):
     with _setup_logging(args):
         return apply(lane.process_alignment, *args)
-process_alignment.metadata = {"resources": ["novoalign", "bwa"]}
+process_alignment.metadata = {"resources": ["novoalign", "bwa", "bowtie", "tophat"]}
 
 @require(lane)
 def align_prep_full(*args):
@@ -125,10 +126,15 @@ def variantcall_sample(*args):
     with _setup_logging(args):
         return apply(genotype.variantcall_sample, *args)
 
-@require(genotype)
+@require(vcfutils)
 def combine_variant_files(*args):
     with _setup_logging(args):
-        return apply(genotype.combine_variant_files, *args)
+        return apply(vcfutils.combine_variant_files, *args)
+
+@require(vcfutils)
+def concat_variant_files(*args):
+    with _setup_logging(args):
+        return apply(vcfutils.concat_variant_files, *args)
 
 @require(population)
 def prep_gemini_db(*args):
@@ -145,3 +151,8 @@ def detect_sv(*args):
 def combine_calls(*args):
     with _setup_logging(args):
         return apply(ensemble.combine_calls, *args)
+
+@require(validate)
+def compare_to_rm(*args):
+    with _setup_logging(args):
+        return apply(validate.compare_to_rm, *args)
