@@ -3,7 +3,8 @@
 from bcbio import utils
 from bcbio.bam import callable
 from bcbio.pipeline import lane, qcsummary, sample, shared, variation
-from bcbio.variation import bamprep, realign, genotype, ensemble, multi, population, recalibrate
+from bcbio.variation import (bamprep, realign, genotype, ensemble, multi, population,
+                             recalibrate, validate, vcfutils)
 
 @utils.map_wrap
 def process_lane(*args):
@@ -16,7 +17,7 @@ def trim_lane(*args):
 @utils.map_wrap
 def process_alignment(*args):
     return lane.process_alignment(*args)
-process_alignment.metadata = {"resources": ["novoalign", "bwa"]}
+process_alignment.metadata = {"resources": ["novoalign", "bwa", "bowtie", "tophat"]}
 
 @utils.map_wrap
 def align_prep_full(*args):
@@ -78,7 +79,11 @@ def variantcall_sample(*args):
 
 @utils.map_wrap
 def combine_variant_files(*args):
-    return genotype.combine_variant_files(*args)
+    return vcfutils.combine_variant_files(*args)
+
+@utils.map_wrap
+def concat_variant_files(*args):
+    return vcfutils.concat_variant_files(*args)
 
 @utils.map_wrap
 def detect_sv(*args):
@@ -100,3 +105,7 @@ def combine_bed(*args):
 @utils.map_wrap
 def calc_callable_loci(*args):
     return callable.calc_callable_loci(*args)
+
+@utils.map_wrap
+def compare_to_rm(*args):
+    return validate.compare_to_rm(*args)
