@@ -14,7 +14,8 @@ import collections
 
 def grouped_parallel_split_combine(args, split_fn, group_fn, parallel_fn,
                                    parallel_name, ungroup_name, combine_name,
-                                   file_key, combine_arg_keys):
+                                   file_key, combine_arg_keys,
+                                   split_outfile_i=-1):
     """Parallel split runner that allows grouping of samples during processing.
 
     This builds on parallel_split_combine to provide the additional ability to
@@ -28,7 +29,8 @@ def grouped_parallel_split_combine(args, split_fn, group_fn, parallel_fn,
       that will pull apart results from grouped analysis into individual sample
       results to combine via `combine_name`
     """
-    split_args, combine_map, finished_out, extras = _get_split_tasks(args, split_fn, file_key)
+    split_args, combine_map, finished_out, extras = _get_split_tasks(args, split_fn, file_key,
+                                                                     split_outfile_i)
     grouped_args, grouped_info = group_fn(split_args)
     split_output = parallel_fn(parallel_name, grouped_args)
     ready_output, grouped_output = _check_group_status(split_output, grouped_info)
@@ -55,7 +57,7 @@ def _check_group_status(xs, grouped_info):
 
 def parallel_split_combine(args, split_fn, parallel_fn,
                            parallel_name, combine_name,
-                           file_key, combine_arg_keys):
+                           file_key, combine_arg_keys, split_outfile_i=-1):
     """Split, run split items in parallel then combine to output file.
 
     split_fn: Split an input file into parts for processing. Returns
@@ -67,8 +69,11 @@ def parallel_split_combine(args, split_fn, parallel_fn,
       bcbio.distributed.tasks/multitasks/ipythontasks to run in parallel.
     combine_name: The name of the function, also from tasks, that combines
       the split output files into a final ready to run file.
+    split_outfile_i: the location of the output file in the arguments
+      generated by the split function. Defaults to the last item in the list.
     """
-    split_args, combine_map, finished_out, extras = _get_split_tasks(args, split_fn, file_key)
+    split_args, combine_map, finished_out, extras = _get_split_tasks(args, split_fn, file_key,
+                                                                     split_outfile_i)
     split_output = parallel_fn(parallel_name, split_args)
     if combine_name:
         combine_args, final_args = _organize_output(split_output, combine_map,
@@ -136,32 +141,56 @@ def _add_combine_extras(args, extras):
     assert added + no_combine_extras >= len(extras), (added, len(extras))
     return out
 
+def _get_extra_args(extra_args, arg_keys):
+    """Retrieve extra arguments to pass along to combine function.
+
+    Special cases like reference files and configuration information
+    are passed as single items, the rest as lists mapping to each data
+    item combined.
+    """
+    # XXX back compatible hack -- should have a way to specify these.
+    single_keys = set(["sam_ref", "config"])
+    out = []
+    for i, arg_key in enumerate(arg_keys):
+        vals = [xs[i] for xs in extra_args]
+        if arg_key in single_keys:
+            out.append(vals[-1])
+        else:
+            out.append(vals)
+    return out
+
 def _organize_output(output, combine_map, file_key, combine_arg_keys):
     """Combine output details for parallelization.
 
     file_key is the key name of the output file used in merging. We extract
     this file from the output data.
+
+    combine_arg_keys are extra items to pass along to the combine function.
     """
     out_map = collections.defaultdict(list)
-    extra_args = {}
+    extra_args = collections.defaultdict(list)
     final_args = {}
     already_added = []
     for data in output:
         cur_file = data[file_key]
         cur_out = combine_map[cur_file]
         out_map[cur_out].append(cur_file)
-        extra_args[cur_out] = [data[x] for x in combine_arg_keys]
+        extra_args[cur_out].append([data[x] for x in combine_arg_keys])
         data[file_key] = cur_out
         if cur_out not in already_added:
             already_added.append(cur_out)
             final_args[cur_out] = data
         elif data.has_key("combine"):
             final_args = _add_combine_parts(final_args, cur_out, data)
-    combine_args = [[v, k] + extra_args[k] for (k, v) in out_map.iteritems()]
+    combine_args = [[v, k] + _get_extra_args(extra_args[k], combine_arg_keys)
+                    for (k, v) in out_map.iteritems()]
     return combine_args, [[final_args[x]] for x in already_added]
 
-def _get_split_tasks(args, split_fn, file_key):
+def _get_split_tasks(args, split_fn, file_key, outfile_i=-1):
     """Split up input files and arguments, returning arguments for parallel processing.
+
+    outfile_i specifies the location of the output file in the arguments to
+    the processing function. Defaults to the last item in the list.
     """
     split_args = []
     combine_map = {}
@@ -172,7 +201,7 @@ def _get_split_tasks(args, split_fn, file_key):
         out_final, out_parts = split_fn(*data)
         for parts in out_parts:
             split_args.append(copy.deepcopy(data) + list(parts))
-        for part_file in [x[-1] for x in out_parts]:
+        for part_file in [x[outfile_i] for x in out_parts]:
             combine_map[part_file] = out_final
         if len(out_parts) == 0:
             if out_final is not None:
