@@ -60,6 +60,18 @@ def can_pipe(fastq_file):
     """
     return True
 
+def check_samtools_version():
+    """Ensure samtools has parallel processing required for piped analysis.
+    """
+    p = subprocess.Popen(["samtools", "sort"], stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
+    output, _ = p.communicate()
+    p.stdout.close()
+    if output.find("-@") == -1:
+        raise OSError("Installed version of samtools sort does not have support for multithreading (-@ option) "
+                      "required to support bwa piped alignment. Please upgrade to the latest version "
+                      "from http://samtools.sourceforge.net/")
+
+
 def align_pipe(fastq_file, pair_file, ref_file, names, align_dir, config):
     """Perform piped alignment of fastq input files, generating sorted output BAM.
     """
@@ -73,6 +85,7 @@ def align_pipe(fastq_file, pair_file, ref_file, names, align_dir, config):
     extra_novo_args = " ".join(_novoalign_args_from_config(config, False))
     rg_info = get_rg_info(names)
     if not utils.file_exists(out_file):
+        check_samtools_version()
         with utils.curdir_tmpdir() as work_dir:
             with file_transaction(out_file) as tx_out_file:
                 tx_out_prefix = os.path.splitext(tx_out_file)[0]
@@ -115,9 +128,10 @@ def _novoalign_args_from_config(config, need_quality=True):
 # paired end sizes
 
 def align(fastq_file, pair_file, ref_file, out_base, align_dir, config,
-          extra_args=None, rg_name=None):
+          extra_args=None, names=None):
     """Align with novoalign.
     """
+    rg_name = names.get("rg", None) if names else None
     out_file = os.path.join(align_dir, "{0}.sam".format(out_base))
     if not file_exists(out_file):
         cl = [config_utils.get_program("novoalign", config)]
@@ -146,6 +160,11 @@ def refindex(ref_file, kmer_size=None, step_size=None, out_file=None):
     cl += [out_file, ref_file]
     subprocess.check_call(cl)
 
+# ## Galaxy integration
+
+# Optional galaxy location file. Falls back on remap_index_fn if not found
+galaxy_location_file = "novoalign_indices.loc"
+
 def remap_index_fn(ref_file):
     """Map sequence references to equivalent novoalign indexes.
     """
