@@ -2,14 +2,17 @@
 
 This works as part of the lane/flowcell process step of the pipeline.
 """
-import os
 from collections import namedtuple
+import ConfigParser
+import os
+from xml.etree import ElementTree
 
 from Bio.SeqIO.QualityIO import FastqGeneralIterator
 
 from bcbio import utils, broad
 from bcbio.bam import cram
-from bcbio.ngsalign import bowtie, bwa, tophat, bowtie2, mosaik, novoalign
+from bcbio.ngsalign import (bowtie, bwa, tophat, bowtie2, mosaik,
+                            novoalign, star)
 from bcbio.distributed.transaction import file_transaction
 
 # Define a next-generation sequencing tool to plugin:
@@ -24,21 +27,21 @@ from bcbio.distributed.transaction import file_transaction
 NgsTool = namedtuple("NgsTool", ["align_fn", "pipe_align_fn", "bam_align_fn",
                                  "galaxy_loc_file", "remap_index_fn", "can_pipe"])
 
-base_location_file = "sam_fa_indices.loc"
+BASE_LOCATION_FILE = "sam_fa_indices.loc"
 _tools = {
     "bowtie": NgsTool(bowtie.align, None, None, bowtie.galaxy_location_file, None, None),
-    "bowtie2": NgsTool(bowtie2.align, None, None, base_location_file, bowtie2.remap_index_fn,
+    "bowtie2": NgsTool(bowtie2.align, None, None, bowtie2.galaxy_location_file, bowtie2.remap_index_fn,
                        None),
     "bwa": NgsTool(bwa.align, bwa.align_pipe, bwa.align_bam, bwa.galaxy_location_file, None,
                    bwa.can_pipe),
     "mosaik": NgsTool(mosaik.align, None, None, mosaik.galaxy_location_file, None,
                       None),
     "novoalign": NgsTool(novoalign.align, novoalign.align_pipe, novoalign.align_bam,
-                         base_location_file, novoalign.remap_index_fn, novoalign.can_pipe),
-    "tophat": NgsTool(tophat.align, None, None, base_location_file, bowtie2.remap_index_fn,
+                         novoalign.galaxy_location_file, novoalign.remap_index_fn, novoalign.can_pipe),
+    "tophat": NgsTool(tophat.align, None, None, bowtie2.galaxy_location_file, bowtie2.remap_index_fn,
                       None),
-    "samtools": NgsTool(None, None, None, base_location_file, None, None),
-    }
+    "samtools": NgsTool(None, None, None, BASE_LOCATION_FILE, None, None),
+    "star": NgsTool(star.align, None, None, None, star.remap_index_fn, None)}
 
 metadata = {"support_bam": [k for k, v in _tools.iteritems() if v.bam_align_fn is not None]}
 
@@ -90,7 +93,7 @@ def _align_from_fastq(fastq1, fastq2, aligner, align_ref, sam_ref, names,
     """
     align_fn = _tools[aligner].align_fn
     sam_file = align_fn(fastq1, fastq2, align_ref, names["lane"], align_dir, config,
-                        rg_name=names["rg"])
+                        names=names)
     if fastq2 is None and aligner in ["bwa", "bowtie2"]:
         fastq1 = _remove_read_number(fastq1, sam_file)
     sort_method = config["algorithm"].get("bam_sort", "coordinate")
@@ -132,7 +135,16 @@ def sam_to_querysort_bam(sam_file, config):
     This allows merging of multiple mappers which do not work with MergeBamAlignment.
     """
     runner = broad.runner_from_config(config)
-    out_file = "{}.bam".format(os.path.splitext(sam_file)[0])
+    out_file = "{}-querysorted.bam".format(os.path.splitext(sam_file)[0])
+    return runner.run_fn("picard_sort", sam_file, "queryname", out_file)
+
+def sam_to_querysort_sam(sam_file, config):
+    """Convert SAM file directly to a query sorted SAM without merging of FASTQ reads.
+
+    This allows merging of multiple mappers which do not work with MergeBamAlignment.
+    """
+    runner = broad.runner_from_config(config)
+    out_file = "{}-querysorted.sam".format(os.path.splitext(sam_file)[0])
     return runner.run_fn("picard_sort", sam_file, "queryname", out_file)
 
 def sam_to_sort_bam(sam_file, ref_file, fastq1, fastq2, sample_name,
@@ -164,35 +176,105 @@ def sam_to_sort_bam(sam_file, ref_file, fastq1, fastq2, sample_name,
             utils.save_diskspace(fastq2, "Merged into output BAM %s" % out_bam, config)
     return sort_bam
 
-def get_genome_ref(genome_build, aligner, galaxy_base):
-    """Retrieve the reference genome file location from galaxy configuration.
+# ## Galaxy integration -- *.loc files
+
+def _get_galaxy_loc_file(name, galaxy_dt, ref_dir, galaxy_base):
+    """Retrieve Galaxy *.loc file for the given reference/aligner name.
+
+    First tries to find an aligner specific *.loc file. If not defined
+    or does not exist, then we need to try and remap it from the
+    default reference file
     """
-    if not genome_build:
-        return (None, None)
-    ref_dir = os.path.join(galaxy_base, "tool-data")
-    out_info = []
-    for ref_get in [aligner, "samtools"]:
-        if not ref_get:
-            out_info.append(None)
-            continue
-        ref_file = os.path.join(ref_dir, _tools[ref_get].galaxy_loc_file)
-        cur_ref = None
-        with open(ref_file) as in_handle:
-            for line in in_handle:
-                if line.strip() and not line.startswith("#"):
-                    parts = line.strip().split()
+    if "file" in galaxy_dt and os.path.exists(os.path.join(galaxy_base, galaxy_dt["file"])):
+        loc_file = os.path.join(galaxy_base, galaxy_dt["file"])
+        need_remap = False
+    elif _tools[name].galaxy_loc_file is None:
+        loc_file = os.path.join(ref_dir, BASE_LOCATION_FILE)
+        need_remap = True
+    else:
+        loc_file = os.path.join(ref_dir, _tools[name].galaxy_loc_file)
+        need_remap = False
+    if not os.path.exists(loc_file):
+        loc_file = os.path.join(ref_dir, BASE_LOCATION_FILE)
+        need_remap = True
+    return loc_file, need_remap
+
+def _get_ref_from_galaxy_loc(name, genome_build, loc_file, galaxy_dt, need_remap):
+    """Retrieve reference genome file from Galaxy *.loc file.
+
+    Reads from tool_data_table_conf.xml information for the index if it
+    exists, otherwise uses heuristics to find line based on most common setups.
+    """
+    if "column" in galaxy_dt:
+        dbkey_i = galaxy_dt["column"].index("dbkey")
+        path_i = galaxy_dt["column"].index("path")
+    else:
+        dbkey_i = None
+    cur_ref = None
+    with open(loc_file) as in_handle:
+        for line in in_handle:
+            if line.strip() and not line.startswith("#"):
+                parts = line.strip().split()
+                if dbkey_i is not None:
+                    if parts[dbkey_i] == genome_build:
+                        cur_ref = parts[path_i]
+                        break
+                else:
                     if parts[0] == "index":
                         parts = parts[1:]
                     if parts[0] == genome_build:
                         cur_ref = parts[-1]
                         break
-        if cur_ref is None:
-            raise IndexError("Genome %s not found in %s" % (genome_build,
-                ref_file))
-        remap_fn = _tools[ref_get].remap_index_fn
-        if remap_fn:
-            cur_ref = remap_fn(cur_ref)
-        out_info.append(utils.add_full_path(cur_ref, ref_dir))
+    if cur_ref is None:
+        raise IndexError("Genome %s not found in %s" % (genome_build, loc_file))
+    if need_remap:
+        remap_fn = _tools[name].remap_index_fn
+        assert remap_fn is not None, "%s requires remapping function from base location file" % name
+        cur_ref = remap_fn(cur_ref)
+    return cur_ref
+
+def _get_galaxy_tool_info(galaxy_base):
+    """Retrieve Galaxy tool-data information from defaults or galaxy config file.
+    """
+    ini_file = os.path.join(galaxy_base, "universe_wsgi.ini")
+    info = {"tool_data_table_config_path": os.path.join(galaxy_base, "tool_data_table_conf.xml"),
+            "tool_data_path": os.path.join(galaxy_base, "tool-data")}
+    config = ConfigParser.ConfigParser()
+    config.read(ini_file)
+    if "app:main" in config.sections():
+        for option in config.options("app:main"):
+            if option in info:
+                info[option] = os.path.join(galaxy_base, config.get("app:main", option))
+    return info
+
+def _get_galaxy_data_table(name, dt_config_file):
+    """Parse data table config file for details on tool *.loc location and columns.
+    """
+    out = {}
+    if os.path.exists(dt_config_file):
+        tdtc = ElementTree.parse(dt_config_file)
+        for t in tdtc.getiterator("table"):
+            if t.attrib.get("name", "") in [name, "%s_indexes" % name]:
+                out["column"] = [x.strip() for x in t.find("columns").text.split(",")]
+                out["file"] = t.find("file").attrib.get("path", "")
+    return out
+
+def get_genome_ref(genome_build, aligner, galaxy_base):
+    """Retrieve the reference genome file location from galaxy configuration.
+    """
+    if not genome_build:
+        return (None, None)
+    galaxy_config = _get_galaxy_tool_info(galaxy_base)
+    out_info = []
+    for name in [aligner, "samtools"]:
+        if not name:
+            out_info.append(None)
+            continue
+        galaxy_dt = _get_galaxy_data_table(name, galaxy_config["tool_data_table_config_path"])
+        loc_file, need_remap = _get_galaxy_loc_file(name, galaxy_dt, galaxy_config["tool_data_path"],
+                                                    galaxy_base)
+        cur_ref = _get_ref_from_galaxy_loc(name, genome_build, loc_file, galaxy_dt, need_remap)
+        out_info.append(utils.add_full_path(cur_ref, galaxy_config["tool_data_path"]))
 
     if len(out_info) != 2:
         raise ValueError("Did not find genome reference for %s %s" %
