@@ -146,9 +146,24 @@ def _get_program_dir(name, config):
 def get_jar(base_name, dname):
     """Retrieve a jar in the provided directory
     """
-    jars = glob.glob(os.path.join(dname, "%s*.jar" % base_name))
+    jars = glob.glob(os.path.join(expand_path(dname), "%s*.jar" % base_name))
     if len(jars) == 1:
         return jars[0]
     else:
         raise ValueError("Could not find java jar %s in %s: %s" % (
             base_name, dname, jars))
+
+## functions for navigating through the standard galaxy directory of files
+
+def get_transcript_gtf(genome_dir):
+    out_file = os.path.join(genome_dir, "rnaseq", "ref-transcripts.gtf")
+    return out_file
+
+def get_rRNA_interval(genome_dir):
+    return os.path.join(genome_dir, "rnaseq", "rRNA.interval")
+
+def get_transcript_refflat(genome_dir):
+    return os.path.join(genome_dir, "rnaseq", "ref-transcripts.refFlat")
+
+def get_rRNA_sequence(genome_dir):
+    return os.path.join(genome_dir, "rnaseq", "rRNA.fa")
