@@ -8,9 +8,13 @@ import sys
 import argparse
 from collections import defaultdict
 
-from bcbio import log, utils, upload
+
+from bcbio import install, log, utils, upload
 from bcbio.bam import callable
+from bcbio.rnaseq import qc
 from bcbio.distributed.messaging import parallel_runner
+from bcbio.distributed.ipython import global_parallel
+from bcbio.log import logger
 from bcbio.pipeline.run_info import get_run_info
 from bcbio.pipeline.demultiplex import add_multiplex_across_lanes
 from bcbio.pipeline.merge import organize_samples
@@ -48,9 +52,9 @@ def run_main(config, config_file, work_dir, parallel,
     pipelines = _pair_lanes_with_pipelines(lane_items)
     for pipeline, pipeline_items in pipelines.items():
         pipeline_items = _add_provenance(pipeline_items, dirs, config)
-        for xs in pipeline.run(config, config_file, run_parallel, dirs, pipeline_items):
-            assert len(xs) == 1
-            upload.from_sample(xs[0])
+        for xs in pipeline.run(config, config_file, run_parallel, parallel, dirs, pipeline_items):
+            if len(xs) == 1:
+                upload.from_sample(xs[0])
     qcsummary.write_metrics(run_info, fc_name, fc_date, dirs)
 
 def _add_provenance(items, dirs, config):
@@ -79,35 +83,57 @@ def parse_cl_args(in_args):
     """
     parser = argparse.ArgumentParser(
         description= "Best-practice pipelines for fully automated high throughput sequencing analysis.")
-    parser.add_argument("global_config", help="Global YAML configuration file specifying details about the system",
-                        nargs="?")
-    parser.add_argument("fc_dir", help="A directory of Illumina output or fastq files to process (optional)",
-                        nargs="?")
-    parser.add_argument("run_config", help="YAML file with details about samples to process (optional)",
-                        nargs="*")
-    parser.add_argument("-n", "--numcores", type=int, default=0)
-    parser.add_argument("-t", "--paralleltype", help="Approach to parallelization",
-                        choices=["local", "ipython", "messaging"], default="local")
-    parser.add_argument("-s", "--scheduler", help="Schedulerto use for ipython parallel",
-                        choices=["lsf", "sge", "torque"])
-    parser.add_argument("-q", "--queue", help="Scheduler queue to run jobs on, for ipython parallel")
-    parser.add_argument("-p", "--profile", help="Profile name to use for ipython parallel",
-                        default="bcbio_nextgen")
-    parser.add_argument("-u", "--upgrade", help="Perform an upgrade of bcbio_nextgen in place.",
-                        choices = ["stable", "development", "system"])
-    parser.add_argument("-w", "--workflow", help="Run a workflow with the given commandline arguments")
-
+    if len(in_args) > 0 and in_args[0] in ["upgrade"]:
+        subparsers = parser.add_subparsers(help="bcbio-nextgen supplemental commands")
+        install.add_subparser(subparsers)
+    else:
+        parser.add_argument("global_config", help="Global YAML configuration file specifying details about the system",
+                            nargs="?")
+        parser.add_argument("fc_dir", help="A directory of Illumina output or fastq files to process (optional)",
+                            nargs="?")
+        parser.add_argument("run_config", help="YAML file with details about samples to process (optional)",
+                            nargs="*")
+        parser.add_argument("-n", "--numcores", type=int, default=0)
+        parser.add_argument("-t", "--paralleltype", help="Approach to parallelization",
+                            choices=["local", "ipython", "messaging"], default="local")
+        parser.add_argument("-s", "--scheduler", help="Schedulerto use for ipython parallel",
+                            choices=["lsf", "sge", "torque", "slurm"])
+        parser.add_argument("-q", "--queue", help="Scheduler queue to run jobs on, for ipython parallel")
+        parser.add_argument("-r", "--resources",
+                            help=("Cluster specific resources specifications. Provide multiple specs separated by ';'\n"
+                                  "Supports SGE: translated to -l parameters"),
+                            default="")
+        parser.add_argument("--timeout", help="Number of minutes before cluster startup times out. Defaults to 15",
+                            default=15, type=int)
+        parser.add_argument("--retries",
+                            help=("Number of retries of failed tasks during distributed processing. "
+                                  "Default 0 (no retries)"),
+                            default=0, type=int)
+        parser.add_argument("-p", "--profile", help="Profile name to use for ipython parallel",
+                            default="bcbio_nextgen")
+        parser.add_argument("-u", "--upgrade", help="Perform an upgrade of bcbio_nextgen in place.",
+                            choices = ["stable", "development", "system"])
+        parser.add_argument("-w", "--workflow", help="Run a workflow with the given commandline arguments")
     args = parser.parse_args(in_args)
-    inputs = [x for x in [args.global_config, args.fc_dir] + args.run_config
-              if x is not None]
-    kwargs = {"numcores": args.numcores if args.numcores > 0 else None,
-              "paralleltype": args.paralleltype,
-              "scheduler": args.scheduler,
-              "queue": args.queue,
-              "profile": args.profile,
-              "upgrade": args.upgrade,
-              "workflow": args.workflow,
-              "inputs": inputs}
+    if hasattr(args, "fc_dir"):
+        inputs = [x for x in [args.global_config, args.fc_dir] + args.run_config
+                  if x is not None]
+        kwargs = {"numcores": args.numcores if args.numcores > 0 else None,
+                  "paralleltype": args.paralleltype,
+                  "scheduler": args.scheduler,
+                  "queue": args.queue,
+                  "timeout": args.timeout,
+                  "retries": args.retries,
+                  "resources": args.resources,
+                  "profile": args.profile,
+                  "upgrade": args.upgrade,
+                  "workflow": args.workflow,
+                  "inputs": inputs}
+    else:
+        args.global_config = None
+        args.fc_dir = None
+        kwargs = {"args": args,
+                  "upgrade": args.upgrade}
     if args.fc_dir is not None and len(args.run_config) == 1:
         kwargs["fc_dir"] = args.fc_dir
         kwargs["run_info_yaml"] = args.run_config[0]
@@ -150,7 +176,7 @@ class AbstractPipeline:
         return
 
     @abc.abstractmethod
-    def run(self, config, config_file, run_parallel, dirs, lanes):
+    def run(self, config, config_file, run_parallel, parallel, dirs, lanes):
         return
 
 
@@ -158,7 +184,7 @@ class VariantPipeline(AbstractPipeline):
     name = "variant"
 
     @classmethod
-    def run(self, config, config_file, run_parallel, dirs, lane_items):
+    def run(self, config, config_file, run_parallel, parallel, dirs, lane_items):
         lane_items = run_parallel("trim_lane", lane_items)
         align_items = run_parallel("process_alignment", lane_items)
         # process samples, potentially multiplexed across multiple lanes
@@ -184,22 +210,46 @@ class Variant2Pipeline(AbstractPipeline):
     name = "variant2"
 
     @classmethod
-    def run(self, config, config_file, run_parallel, dirs, lane_items):
-        # Handle alignment and preparation requiring the entire input file
-        samples = run_parallel("align_prep_full", (list(x) + [config_file] for x in lane_items))
-        regions = callable.combine_sample_regions(samples)
-        samples = region.add_region_info(samples, regions)
-        # Handle all variant calling on sub-regions of the input file
-        samples = region.clean_sample_data(samples)
-        samples = region.parallel_prep_region(samples, regions, run_parallel)
-        samples = region.parallel_variantcall_region(samples, run_parallel)
-        samples = run_parallel("postprocess_variants", samples)
-        samples = combine_multiple_callers(samples)
-        samples = ensemble.combine_calls_parallel(samples, run_parallel)
-        samples = population.prep_db_parallel(samples, run_parallel)
-        samples = region.delayed_bamprep_merge(samples, run_parallel)
-        samples = qcsummary.generate_parallel(samples, run_parallel)
-        samples = validate.summarize_grading(samples)
+    def run(self, config, config_file, run_parallel, parallel, dirs, lane_items):
+        ## Alignment and preparation requiring the entire input file (multicore cluster)
+        with global_parallel(parallel, "multicore", ["align_prep_full"],
+                             lane_items, dirs["work"], config) as parallel:
+            run_parallel = parallel_runner(parallel, dirs, config)
+            logger.info("Timing: alignment")
+            samples = run_parallel("align_prep_full", [list(x) + [config_file] for x in lane_items])
+            regions = callable.combine_sample_regions(samples)
+            samples = region.add_region_info(samples, regions)
+            samples = region.clean_sample_data(samples)
+
+        ## Variant calling on sub-regions of the input file (full cluster)
+        with global_parallel(parallel, "full", ["piped_bamprep", "variantcall_sample"],
+                             samples, dirs["work"], config,
+                             multiplier=len(regions["analysis"])) as parallel:
+            run_parallel = parallel_runner(parallel, dirs, config)
+            logger.info("Timing: alignment post-processing")
+            samples = region.parallel_prep_region(samples, regions, run_parallel)
+            logger.info("Timing: variant calling")
+            samples = region.parallel_variantcall_region(samples, run_parallel)
+
+        ## Finalize variants (per-sample cluster)
+        with global_parallel(parallel, "persample", ["postprocess_variants"],
+                             samples, dirs["work"], config) as parallel:
+            run_parallel = parallel_runner(parallel, dirs, config)
+            logger.info("Timing: variant post-processing")
+            samples = run_parallel("postprocess_variants", samples)
+            samples = combine_multiple_callers(samples)
+            logger.info("Timing: ensemble calling")
+            samples = ensemble.combine_calls_parallel(samples, run_parallel)
+            logger.info("Timing: prepped BAM merging")
+            samples = region.delayed_bamprep_merge(samples, run_parallel)
+            logger.info("Timing: validation")
+            samples = run_parallel("compare_to_rm", samples)
+            samples = validate.summarize_grading(samples)
+            logger.info("Timing: population database")
+            samples = population.prep_db_parallel(samples, run_parallel)
+            logger.info("Timing: quality control")
+            samples = qcsummary.generate_parallel(samples, run_parallel)
+            logger.info("Timing: finished")
         return samples
 
 class SNPCallingPipeline(VariantPipeline):
@@ -217,15 +267,16 @@ class RnaseqPipeline(AbstractPipeline):
     name = "RNA-seq"
 
     @classmethod
-    def run(self, config, config_file, run_parallel, dirs, lane_items):
+    def run(self, config, config_file, run_parallel, parallel, dirs, lane_items):
         lane_items = run_parallel("trim_lane", lane_items)
         align_items = run_parallel("process_alignment", lane_items)
         # process samples, potentially multiplexed across multiple lanes
         samples = organize_samples(align_items, dirs, config_file)
         samples = run_parallel("merge_sample", samples)
         samples = run_parallel("generate_transcript_counts", samples)
-        run_parallel("generate_bigwig", samples, {"programs": ["ucsc_bigwig"]})
         samples = qcsummary.generate_parallel(samples, run_parallel)
+        samples = qc.sample_summary(samples)
+        #run_parallel("generate_bigwig", samples, {"programs": ["ucsc_bigwig"]})
         return samples
 
 def _get_pipeline(lane_item):
