@@ -7,7 +7,7 @@ from bcbio.log import logger
 from bcbio.pipeline.shared import configured_vrn_files
 from bcbio.structural import hydra
 from bcbio.variation.genotype import variant_filtration
-from bcbio.variation import annotation, effects, validate
+from bcbio.variation import annotation, effects
 
 # ## Genotyping
 
@@ -15,15 +15,13 @@ def postprocess_variants(data):
     """Provide post-processing of variant calls.
     """
     logger.info("Finalizing variant calls: %s" % str(data["name"]))
-    if data["work_bam"]:
+    if data["work_bam"] and data.get("vrn_file"):
         data["vrn_file"] = finalize_genotyper(data["vrn_file"], data["work_bam"],
                                               data["sam_ref"], data["config"])
         logger.info("Calculating variation effects for %s" % str(data["name"]))
-        ann_vrn_file = effects.snpeff_effects(data["vrn_file"], data["genome_build"],
-                                              data["config"])
+        ann_vrn_file = effects.snpeff_effects(data)
         if ann_vrn_file:
             data["vrn_file"] = ann_vrn_file
-    data = validate.compare_to_rm(data)
     return [[data]]
 
 def finalize_genotyper(call_file, bam_file, ref_file, config):
