@@ -15,7 +15,7 @@ from bcbio import utils
 from bcbio.bam import callable
 from bcbio.log import logger
 from bcbio.pipeline import config_utils
-from bcbio.provenance.diagnostics import log_cmd
+from bcbio.provenance import do
 
 def _has_ensemble(data):
     return len(data["variants"]) > 1 and data["config"]["algorithm"].has_key("ensemble")
@@ -59,7 +59,7 @@ def bcbio_variation_comparison(config_file, base_dir, data):
     jvm_opts = resources.get("jvm_opts", ["-Xms750m", "-Xmx2g"])
     java_args = ["-Djava.io.tmpdir=%s" % tmp_dir]
     cmd = ["java"] + jvm_opts + java_args + ["-jar", bv_jar, "variant-compare", config_file]
-    log_cmd("Comparing variant calls using bcbio.variation", data, " ".join(cmd))
+    do.run(cmd, "Comparing variant calls using bcbio.variation", data)
     subprocess.check_call(cmd)
 
 def _run_bcbio_variation(config_file, base_dir, sample, data):
