@@ -35,10 +35,11 @@ def _freebayes_options_from_config(aconfig, out_file, region=None):
         opts += ["--variant-input", background]
     return opts
 
-def run_freebayes(align_bams, ref_file, config, dbsnp=None, region=None,
+def run_freebayes(align_bams, items, ref_file, assoc_files, region=None,
                   out_file=None):
     """Detect SNPs and indels with FreeBayes.
     """
+    config = items[0]["config"]
     broad_runner = broad.runner_from_config(config)
     if out_file is None:
         out_file = "%s-variants.vcf" % os.path.splitext(align_bams[0])[0]
