@@ -19,10 +19,10 @@ from bcbio.log import logger
 from bcbio.utils import file_exists, safe_makedir
 from bcbio.distributed.transaction import file_transaction
 from bcbio.distributed.split import grouped_parallel_split_combine
-from bcbio.pipeline.shared import (process_bam_by_chromosome, configured_ref_file,
+from bcbio.pipeline.shared import (process_bam_by_chromosome, configured_vrn_files,
                                    subset_variant_regions)
 from bcbio.variation.realign import has_aligned_reads
-from bcbio.variation import annotation, bamprep, multi, phasing
+from bcbio.variation import annotation, bamprep, multi, phasing, vcfutils
 
 # ## GATK Genotype calling
 
@@ -54,7 +54,7 @@ def _shared_gatk_call_prep(align_bams, ref_file, config, dbsnp, region, out_file
         params += ["-L", bamprep.region_to_gatk(region), "--interval_set_rule", "INTERSECTION"]
     return broad_runner, params
 
-def unified_genotyper(align_bams, ref_file, config, dbsnp=None,
+def unified_genotyper(align_bams, items, ref_file, assoc_files,
                        region=None, out_file=None):
     """Perform SNP genotyping on the given alignment file.
     """
@@ -62,7 +62,7 @@ def unified_genotyper(align_bams, ref_file, config, dbsnp=None,
         out_file = "%s-variants.vcf" % os.path.splitext(align_bams[0])[0]
     if not file_exists(out_file):
         broad_runner, params = \
-            _shared_gatk_call_prep(align_bams, ref_file, config, dbsnp,
+            _shared_gatk_call_prep(align_bams, ref_file, items[0]["config"], assoc_files.dbsnp,
                                    region, out_file)
         if (not isinstance(region, (list, tuple)) and
                 not all(has_aligned_reads(x, region) for x in align_bams)):
@@ -75,7 +75,7 @@ def unified_genotyper(align_bams, ref_file, config, dbsnp=None,
                 broad_runner.run_gatk(params)
     return out_file
 
-def haplotype_caller(align_bams, ref_file, config, dbsnp=None,
+def haplotype_caller(align_bams, items, ref_file, assoc_files,
                        region=None, out_file=None):
     """Call variation with GATK's HaplotypeCaller.
 
@@ -85,7 +85,7 @@ def haplotype_caller(align_bams, ref_file, config, dbsnp=None,
         out_file = "%s-variants.vcf" % os.path.splitext(align_bams[0])[0]
     if not file_exists(out_file):
         broad_runner, params = \
-            _shared_gatk_call_prep(align_bams, ref_file, config, dbsnp,
+            _shared_gatk_call_prep(align_bams, ref_file, items[0]["config"], assoc_files.dbsnp,
                                    region, out_file)
         assert broad_runner.gatk_type() == "restricted", \
             "Require full version of GATK 2.4+ for haplotype calling"
@@ -136,55 +136,17 @@ def write_empty_vcf(out_file):
                          "## No variants; no reads aligned in region\n"
                          "#CHROM\tPOS\tID\tREF\tALT\tQUAL\tFILTER\tINFO\n")
 
-# ## Utility functions for dealing with VCF files
-
-def split_snps_indels(broad_runner, orig_file, ref_file):
-    """Split a variant call file into SNPs and INDELs for processing.
-    """
-    base, ext = os.path.splitext(orig_file)
-    snp_file = "{base}-snp{ext}".format(base=base, ext=ext)
-    indel_file = "{base}-indel{ext}".format(base=base, ext=ext)
-    params = ["-T", "SelectVariants",
-              "-R", ref_file,
-              "--variant", orig_file]
-    for out_file, select_type in [(snp_file, ["SNP"]),
-                                  (indel_file, ["INDEL", "MIXED", "MNP",
-                                                "SYMBOLIC", "NO_VARIATION"])]:
-        if not file_exists(out_file):
-            with file_transaction(out_file) as tx_out_file:
-                cur_params = params + ["--out", tx_out_file]
-                for x in select_type:
-                    cur_params += ["--selectTypeToInclude", x]
-                broad_runner.run_gatk(cur_params)
-    return snp_file, indel_file
-
-def combine_variant_files(orig_files, out_file, ref_file, config,
-                          quiet_out=True):
-    """Combine multiple VCF files into a single output file.
-    """
-    broad_runner = broad.runner_from_config(config)
-    if not file_exists(out_file):
-        with file_transaction(out_file) as tx_out_file:
-            params = ["-T", "CombineVariants",
-                      "-R", ref_file,
-                      "--out", tx_out_file]
-            priority_order = []
-            for i, orig_file in enumerate(orig_files):
-                name = "v%s" % i
-                params.extend(["--variant:{name}".format(name=name), orig_file])
-                priority_order.append(name)
-            params.extend(["--rod_priority_list", ",".join(priority_order)])
-            if quiet_out:
-                params.extend(["--suppressCommandLineHeader", "--setKey", "null"])
-            variant_regions = config["algorithm"].get("variant_regions", None)
-            if variant_regions:
-                params += ["-L", bamprep.region_to_gatk(variant_regions),
-                           "--interval_set_rule", "INTERSECTION"]
-            broad_runner.run_gatk(params)
-    return out_file
-
 # ## Variant filtration -- shared functionality
 
+def _get_coverage_params(config):
+    Cov = collections.namedtuple("Cov", ["interval", "depth"])
+    return Cov(config["algorithm"].get("coverage_interval", "exome").lower(),
+               config["algorithm"].get("coverage_depth", "high").lower())
+
+def _no_vqsr(config):
+    cov = _get_coverage_params(config)
+    return cov.interval in ["regional", "exome"] or cov.depth == "low"
+
 def variant_filtration(call_file, ref_file, vrn_files, config):
     """Filter variant calls using Variant Quality Score Recalibration.
 
@@ -192,8 +154,7 @@ def variant_filtration(call_file, ref_file, vrn_files, config):
     """
     broad_runner = broad.runner_from_config(config)
     caller = config["algorithm"].get("variantcaller")
-    cov_interval = config["algorithm"].get("coverage_interval", "exome").lower()
-    if caller in ["gatk-haplotype"] and cov_interval not in ["regional"]:
+    if caller in ["gatk-haplotype"] and not _no_vqsr(config):
         return _variant_filtration_both(broad_runner, call_file, ref_file, vrn_files,
                                         config)
     elif caller in ["freebayes"]:
@@ -202,14 +163,14 @@ def variant_filtration(call_file, ref_file, vrn_files, config):
     elif caller in ["samtools", "varscan"]:
         return call_file
     else:
-        snp_file, indel_file = split_snps_indels(broad_runner, call_file, ref_file)
+        snp_file, indel_file = vcfutils.split_snps_indels(broad_runner, call_file, ref_file)
         snp_filter_file = _variant_filtration_snp(broad_runner, snp_file, ref_file,
                                                   vrn_files, config)
         indel_filter_file = _variant_filtration_indel(broad_runner, indel_file,
                                                       ref_file, vrn_files, config)
         orig_files = [snp_filter_file, indel_filter_file]
         out_file = "{base}combined.vcf".format(base=os.path.commonprefix(orig_files))
-        return combine_variant_files(orig_files, out_file, ref_file, config)
+        return vcfutils.combine_variant_files(orig_files, out_file, ref_file, config)
 
 def filter_freebayes(broad_runner, in_file, ref_file, vrn_files, config):
     """Perform basic sanity filtering of FreeBayes results, removing low confidence calls.
@@ -236,8 +197,7 @@ def _apply_variant_recal(broad_runner, snp_file, ref_file, recal_file,
             broad_runner.run_gatk(params)
     return out_file
 
-def _shared_variant_filtration(filter_type, cov_interval, snp_file,
-                               ref_file, vrn_files):
+def _shared_variant_filtration(filter_type, snp_file, ref_file, vrn_files):
     """Share functionality for filtering variants.
     """
     recal_file = "{base}.recal".format(base=os.path.splitext(snp_file)[0])
@@ -266,10 +226,7 @@ def _shared_variant_filtration(filter_type, cov_interval, snp_file,
         params.extend(
             ["-resource:mills,VCF,known=true,training=true,truth=true,prior=12.0",
              vrn_files.train_indels])
-    if cov_interval == "exome":
-        params.extend(["--maxGaussians", "4", "--percentBadVariants", "0.05"])
-    else:
-        params.extend(["-an", "DP"])
+    params.extend(["-an", "DP"])
     return params, recal_file, tranches_file
 
 def variant_filtration_with_exp(broad_runner, snp_file, ref_file, filter_type,
@@ -304,10 +261,9 @@ def _variant_filtration_snp(broad_runner, snp_file, ref_file, vrn_files,
     """Filter SNP variant calls using GATK best practice recommendations.
     """
     filter_type = "SNP"
-    cov_interval = config["algorithm"].get("coverage_interval", "exome").lower()
     variantcaller = config["algorithm"].get("variantcaller", "gatk")
     params, recal_file, tranches_file = _shared_variant_filtration(
-        filter_type, cov_interval, snp_file, ref_file, vrn_files)
+        filter_type, snp_file, ref_file, vrn_files)
     assert vrn_files.train_hapmap and vrn_files.train_1000g_omni, \
            "Need HapMap and 1000 genomes training files"
     filters = ["QD < 2.0", "MQ < 40.0", "FS > 60.0",
@@ -316,7 +272,7 @@ def _variant_filtration_snp(broad_runner, snp_file, ref_file, vrn_files,
     # resulting in excessive filtering, so avoid this metric
     if variantcaller not in ["gatk-haplotype"]:
         filters.append("HaplotypeScore > 13.0")
-    if cov_interval == "regional" or variantcaller == "freebayes":
+    if _no_vqsr(config):
         return variant_filtration_with_exp(broad_runner, snp_file, ref_file, filter_type,
                                            filters)
     else:
@@ -348,10 +304,9 @@ def _variant_filtration_indel(broad_runner, snp_file, ref_file, vrn_files,
     """Filter indel variant calls using GATK best practice recommendations.
     """
     filter_type = "INDEL"
-    cov_interval = config["algorithm"].get("coverage_interval", "exome").lower()
     params, recal_file, tranches_file = _shared_variant_filtration(
-        filter_type, cov_interval, snp_file, ref_file, vrn_files)
-    if cov_interval in ["exome", "regional"]:
+        filter_type, snp_file, ref_file, vrn_files)
+    if _no_vqsr(config):
         return variant_filtration_with_exp(broad_runner, snp_file, ref_file, filter_type,
                                            ["QD < 2.0", "ReadPosRankSum < -20.0", "FS > 200.0"])
     else:
@@ -370,9 +325,8 @@ def _variant_filtration_both(broad_runner, snp_file, ref_file, vrn_files,
     """Filter SNP and indel variant calls using GATK best practice recommendations.
     """
     filter_type = "BOTH"
-    cov_interval = config["algorithm"].get("coverage_interval", "exome").lower()
     params, recal_file, tranches_file = _shared_variant_filtration(
-        filter_type, cov_interval, snp_file, ref_file, vrn_files)
+        filter_type, snp_file, ref_file, vrn_files)
     if not file_exists(recal_file):
         with file_transaction(recal_file, tranches_file) as (tx_recal, tx_tranches):
             params.extend(["--recal_file", tx_recal,
@@ -548,24 +502,27 @@ def parallel_variantcall(sample_info, parallel_fn):
 def variantcall_sample(data, region=None, out_file=None):
     """Parallel entry point for doing genotyping of a region of a sample.
     """
-    from bcbio.variation import freebayes, cortex, samtools, varscan
+    from bcbio.variation import freebayes, cortex, samtools, varscan, mutect
     safe_makedir(os.path.dirname(out_file))
     caller_fns = {"gatk": unified_genotyper,
                   "gatk-haplotype": haplotype_caller,
                   "freebayes": freebayes.run_freebayes,
                   "cortex": cortex.run_cortex,
                   "samtools": samtools.run_samtools,
-                  "varscan": varscan.run_varscan}
+                  "varscan": varscan.run_varscan,
+                  "mutect": mutect.mutect_caller}
     sam_ref = data["sam_ref"]
     config = data["config"]
     caller_fn = caller_fns[config["algorithm"].get("variantcaller", "gatk")]
     if isinstance(data["work_bam"], basestring):
         align_bams = [data["work_bam"]]
+        items = [data]
     else:
         align_bams = data["work_bam"]
+        items = data["work_items"]
     call_file = "%s-raw%s" % os.path.splitext(out_file)
-    caller_fn(align_bams, sam_ref, config,
-              configured_ref_file("dbsnp", config, sam_ref),
+    caller_fn(align_bams, items, sam_ref,
+              configured_vrn_files(config, sam_ref),
               region, call_file)
     if data["config"]["algorithm"].get("phasing", False) == "gatk":
         call_file = phasing.read_backed_phasing(call_file, align_bams, sam_ref, region, config)
@@ -573,5 +530,7 @@ def variantcall_sample(data, region=None, out_file=None):
         for ext in ["", ".idx"]:
             if os.path.exists(call_file + ext):
                 os.symlink(call_file + ext, out_file + ext)
+    if "work_items" in data:
+        del data["work_items"]
     data["vrn_file"] = out_file
     return [data]
